@@ -151,12 +151,20 @@ should_use_https_everywhere (const char *request_uri,
 
 static gboolean
 should_use_adblocker (const char *request_uri,
-                      const char *page_uri)
+                      const char *page_uri,
+                      const char *redirected_request_uri)
 {
-  /* Always load the main resource. */
+  if (!g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
+    return FALSE;
+
+  /* Always load the main resource... */
   if (g_strcmp0 (request_uri, page_uri) == 0)
     return FALSE;
 
+  /* ...even during a redirect, when page_uri is stale. */
+  if (g_strcmp0 (page_uri, redirected_request_uri) == 0)
+    return FALSE;
+
   /* Always load data requests, as uri_tester won't do any good here. */
   if (g_str_has_prefix (request_uri, SOUP_URI_SCHEME_DATA))
     return FALSE;
@@ -192,9 +200,9 @@ web_page_send_request (WebKitWebPage     *web_page,
 
   request_uri = webkit_uri_request_get_uri (request);
   page_uri = webkit_web_page_get_uri (web_page);
+  redirected_response_uri = redirected_response ? webkit_uri_response_get_uri (redirected_response) : NULL;
 
-  if (!should_use_adblocker (request_uri, page_uri) ||
-      !g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
+  if (!should_use_adblocker (request_uri, page_uri, redirected_response_uri))
     flags &= ~EPHY_URI_TEST_ADBLOCK;
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
@@ -207,7 +215,6 @@ web_page_send_request (WebKitWebPage     *web_page,
     modified_uri = ephy_remove_tracking_from_uri (request_uri);
   }
 
-  redirected_response_uri = redirected_response ? webkit_uri_response_get_uri (redirected_response) : NULL;
   if (!should_use_https_everywhere (request_uri, redirected_response_uri))
     flags &= ~EPHY_URI_TEST_HTTPS_EVERYWHERE;
 
@@ -234,6 +241,7 @@ web_page_send_request (WebKitWebPage     *web_page,
     LOG ("Rewrote %s to %s", request_uri, modified_uri);
     webkit_uri_request_set_uri (request, modified_uri);
   }
+
   g_free (modified_uri);
 
   return FALSE;
@@ -266,14 +274,18 @@ store_password (EphyEmbedFormAuth *form_auth)
 {
   SoupURI *uri;
   char *uri_str;
+  char *origin;
   char *username_field_name = NULL;
   char *username_field_value = NULL;
   char *password_field_name = NULL;
   char *password_field_value = NULL;
+  const char *target_origin;
   gboolean password_updated;
   WebKitDOMNode *username_node;
   EphyWebExtension *extension = ephy_web_extension_get ();
 
+  g_assert (extension->password_manager);
+
   username_node = ephy_embed_form_auth_get_username_node (form_auth);
   if (username_node)
     g_object_get (username_node,
@@ -287,9 +299,12 @@ store_password (EphyEmbedFormAuth *form_auth)
 
   uri = ephy_embed_form_auth_get_uri (form_auth);
   uri_str = soup_uri_to_string (uri, FALSE);
+  origin = ephy_uri_to_security_origin (uri_str);
+  target_origin = ephy_embed_form_auth_get_target_origin (form_auth);
   password_updated = ephy_embed_form_auth_get_password_updated (form_auth);
   ephy_password_manager_save (extension->password_manager,
-                              uri_str,
+                              origin,
+                              target_origin,
                               username_field_value,
                               password_field_value,
                               username_field_name,
@@ -297,6 +312,7 @@ store_password (EphyEmbedFormAuth *form_auth)
                               !password_updated);
 
   g_free (uri_str);
+  g_free (origin);
   g_free (username_field_name);
   g_free (username_field_value);
   g_free (password_field_name);
@@ -456,14 +472,23 @@ form_submitted_cb (WebKitDOMHTMLFormElement *dom_form,
   EphyWebExtension *extension = ephy_web_extension_get ();
   EphyEmbedFormAuth *form_auth;
   SoupURI *uri;
+  char *target_origin;
   WebKitDOMNode *username_node = NULL;
   WebKitDOMNode *password_node = NULL;
   char *username_field_name = NULL;
   char *username_field_value = NULL;
   char *password_field_name = NULL;
   char *uri_str;
+  char *origin;
+  char *form_action;
 
-  if (!ephy_web_dom_utils_find_form_auth_elements (dom_form, &username_node, &password_node))
+  if (!extension->password_manager)
+    return TRUE;
+
+  if (!ephy_web_dom_utils_find_form_auth_elements (dom_form,
+                                                   &username_node,
+                                                   &password_node,
+                                                   AUTH_CACHE_SUBMIT))
     return TRUE;
 
   if (username_node) {
@@ -472,8 +497,17 @@ form_submitted_cb (WebKitDOMHTMLFormElement *dom_form,
                   NULL);
   }
 
+  form_action = webkit_dom_html_form_element_get_action (dom_form);
+  if (form_action == NULL)
+    form_action = g_strdup (webkit_web_page_get_uri (web_page));
+  target_origin = ephy_uri_to_security_origin (form_action);
+
   /* EphyEmbedFormAuth takes ownership of the nodes */
-  form_auth = ephy_embed_form_auth_new (web_page, username_node, password_node, username_field_value);
+  form_auth = ephy_embed_form_auth_new (web_page,
+                                        target_origin,
+                                        username_node,
+                                        password_node,
+                                        username_field_value);
   uri = ephy_embed_form_auth_get_uri (form_auth);
   soup_uri_set_query (uri, NULL);
 
@@ -481,20 +515,25 @@ form_submitted_cb (WebKitDOMHTMLFormElement *dom_form,
     g_object_get (username_node, "name", &username_field_name, NULL);
   g_object_get (password_node, "name", &password_field_name, NULL);
   uri_str = soup_uri_to_string (uri, FALSE);
+  origin = ephy_uri_to_security_origin (uri_str);
 
   ephy_password_manager_query (extension->password_manager,
                                NULL,
-                               uri_str,
+                               origin,
+                               target_origin,
                                username_field_value,
                                username_field_name,
                                password_field_name,
                                should_store_cb,
                                form_auth);
 
+  g_free (form_action);
+  g_free (target_origin);
   g_free (username_field_name);
   g_free (username_field_value);
   g_free (password_field_name);
   g_free (uri_str);
+  g_free (origin);
 
   return TRUE;
 }
@@ -537,9 +576,11 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
 {
   SoupURI *uri;
   char *uri_str;
+  char *origin;
   char *username = NULL;
   char *username_field_name = NULL;
   char *password_field_name = NULL;
+  const char *target_origin;
   WebKitDOMNode *username_node;
   WebKitDOMNode *password_node;
   EphyWebExtension *extension;
@@ -549,7 +590,9 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
     return;
 
   extension = ephy_web_extension_get ();
-  uri_str = soup_uri_to_string (uri, FALSE);
+  if (!extension->password_manager)
+    return;
+
   username_node = ephy_embed_form_auth_get_username_node (form_auth);
   if (username_node) {
     g_object_get (username_node, "name", &username_field_name, NULL);
@@ -563,9 +606,14 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
   if (username != NULL && g_str_equal (username, ""))
     g_clear_pointer (&username, g_free);
 
+  uri_str = soup_uri_to_string (uri, FALSE);
+  origin = ephy_uri_to_security_origin (uri_str);
+  target_origin = ephy_embed_form_auth_get_target_origin (form_auth);
+
   ephy_password_manager_query (extension->password_manager,
                                NULL,
-                               uri_str,
+                               origin,
+                               target_origin,
                                username,
                                username_field_name,
                                password_field_name,
@@ -573,6 +621,7 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
                                form_auth);
 
   g_free (uri_str);
+  g_free (origin);
   g_free (username);
   g_free (username_field_name);
   g_free (password_field_name);
@@ -1111,15 +1160,32 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
       continue;
 
     /* We have a field that may be the user, and one for a password. */
-    if (ephy_web_dom_utils_find_form_auth_elements (form, &username_node, &password_node)) {
+    if (ephy_web_dom_utils_find_form_auth_elements (form,
+                                                    &username_node,
+                                                    &password_node,
+                                                    AUTH_CACHE_AUTOFILL)) {
       EphyEmbedFormAuth *form_auth;
       GList *cached_users;
       const char *uri;
+      char *origin;
+      char *form_action;
+      char *target_origin;
+
+      uri = webkit_web_page_get_uri (web_page);
+
+      form_action = webkit_dom_html_form_element_get_action (form);
+      if (form_action == NULL)
+        form_action = g_strdup (uri);
+      target_origin = ephy_uri_to_security_origin (form_action);
 
       LOG ("Hooking and pre-filling a form");
 
       /* EphyEmbedFormAuth takes ownership of the nodes */
-      form_auth = ephy_embed_form_auth_new (web_page, username_node, password_node, NULL);
+      form_auth = ephy_embed_form_auth_new (web_page,
+                                            target_origin,
+                                            username_node,
+                                            password_node,
+                                            NULL);
       webkit_dom_event_target_add_event_listener (WEBKIT_DOM_EVENT_TARGET (form), "submit",
                                                   G_CALLBACK (form_submitted_cb), FALSE,
                                                   web_page);
@@ -1130,8 +1196,8 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
       }
 
       /* Plug in the user autocomplete */
-      uri = webkit_web_page_get_uri (web_page);
-      cached_users = ephy_password_manager_get_cached_users_for_uri (extension->password_manager, uri);
+      origin = ephy_uri_to_security_origin (uri);
+      cached_users = ephy_password_manager_get_cached_users (extension->password_manager, origin);
 
       if (cached_users && cached_users->next && username_node) {
         LOG ("More than 1 password saved, hooking menu for choosing which on focus");
@@ -1158,6 +1224,9 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
 
       pre_fill_form (form_auth);
 
+      g_free (origin);
+      g_free (form_action);
+      g_free (target_origin);
       g_object_weak_ref (G_OBJECT (form), form_destroyed_cb, form_auth);
     } else
       LOG ("No pre-fillable/hookable form found");
