@@ -2,22 +2,23 @@
 /*
  *  Copyright © 2003 Christian Persch
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef EPHY_ZOOM_H
-#define EPHY_ZOOM_H
+#pragma once
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -54,11 +55,10 @@ static const guint n_zoom_levels = G_N_ELEMENTS (zoom_levels);
 #define ZOOM_IN   (-1.0)
 #define ZOOM_OUT  (-2.0)
 
-int   ephy_zoom_get_zoom_level_index   (float level);
-float ephy_zoom_get_changed_zoom_level (float level,
-                                        int   steps);
-float ephy_zoom_get_nearest_zoom_level (float level);
+int         ephy_zoom_get_zoom_level_index   (float level);
+float       ephy_zoom_get_changed_zoom_level (float level,
+                                              int   steps);
+float       ephy_zoom_get_nearest_zoom_level (float level);
+const char *ephy_zoom_get_zoom_level_name    (float level);
 
 G_END_DECLS
-
-#endif
