@@ -51,7 +51,7 @@ struct _EphyLocationController {
   EphyWindow *window;
   EphyTitleWidget *title_widget;
   char *address;
-  guint editable : 1;
+  gboolean editable;
   gboolean sync_address_is_blocked;
   EphySearchEngineManager *search_engine_manager;
   guint num_search_engines_actions;
@@ -227,11 +227,12 @@ get_location_cb (EphyLocationEntry      *entry,
                  EphyLocationController *controller)
 {
   EphyEmbed *embed;
+  const char *address;
 
-  embed = ephy_embed_container_get_active_child
-            (EPHY_EMBED_CONTAINER (controller->window));
+  embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (controller->window));
+  address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
-  return g_strdup (ephy_web_view_get_address (ephy_embed_get_web_view (embed)));
+  return ephy_embed_utils_is_no_show_address (address) ? NULL : g_strdup (address);
 }
 
 static char *
@@ -406,8 +407,8 @@ ephy_location_controller_constructed (GObject *object)
 
   add_completion_actions (controller, EPHY_LOCATION_ENTRY (controller->title_widget));
 
-  g_signal_connect (controller->search_engine_manager, "changed",
-                    G_CALLBACK (search_engines_changed_cb), controller);
+  g_signal_connect_object (controller->search_engine_manager, "changed",
+                           G_CALLBACK (search_engines_changed_cb), controller, 0);
 
   g_object_bind_property (controller, "editable",
                           controller->title_widget, "editable",
